@@ -121,7 +121,9 @@ static int _idxvalfree(v)
 {
   if( !v )       return 0;
   if( v->s )     xfree(v->s);
+#ifdef HAVE_SYS_MMAN_H
   if( v->idata ) munmap(v->idata, v->ilen);
+#endif
   /* these are linked (i.e. if iname does not exist, its a copy of another */
   if( v->iname && v->igio && v->ifits ){
     gclose(v->igio);
@@ -153,6 +155,7 @@ static int _idxrowfree(r)
     close(r->ichan);
   if( r->ochan )
     close(r->ochan);
+#if HAVE_MINGW32==0
   if( r->pid ){
     while( (waitpid(r->pid, &exit_status, WNOHANG)==0) && (tries<10) ){
       gsleep(10);
@@ -160,8 +163,11 @@ static int _idxrowfree(r)
     }
     r->pid = 0;
   }
+#endif
   if( r->fdata ){
+#ifdef HAVE_SYS_MMAN_H
     munmap(r->fdata, r->flen);
+#endif
   }
   xfree(r);
   return 1;
@@ -680,6 +686,7 @@ static idxrowrec *_idxrowmerge(x, row1, row2, type)
   int obuf[SZ_LINE];
   FILE *ifd1, *ifd2, *ofd;
 
+#if HAVE_MINGW32==0
   /* we fork a process that will read the output from 2 instances of sort and
      either OR and AND the results. Doing this avoids having the main process
      read the output of the sorts and write to a new merge process */
@@ -802,6 +809,9 @@ static idxrowrec *_idxrowmerge(x, row1, row2, type)
     fflush(ofd);
     _exit(0);
   }
+#else
+  idxerror("can't use index optimization with MinGW");
+#endif
   /* shouldn't get here */
   return NULL;
 }
@@ -1445,7 +1455,7 @@ idxrowrec *idxrowreg(idxvalrec *val)
 }
 
 #ifdef ANSI_FUNC
-idxrowrec *idxrowfun(idxvalrec *val)
+idxrowrec *idxrowfun(idxvalrec *UNUSED(val))
 #else
 idxrowrec *idxrowfun(idxvalrec *val)
   idxvalrec *val;
@@ -1453,9 +1463,6 @@ idxrowrec *idxrowfun(idxvalrec *val)
 {
   idxrowrec *x=NULL;
 
-  /* avoid -W unused parameter warning */
-  if( 0 ) val = val;
-
   x = idxrownew();
   x->type = IDX_INDEF;
   x->rtype = IDX_INDEF;
@@ -2359,4 +2366,3 @@ int idxdebug(debug)
   idx_debug = debug;
   return odebug;
 }
-
