@@ -112,7 +112,7 @@ main(argc, argv)
   int tltyp[2];
   int bin[2];
   double tlmin[2];
-  double tlmax[2];
+  //  double tlmax[2];
   double binsiz[2];
   double xtot[MAXREGIONS+1], ytot[MAXREGIONS+1];
   double xavg[MAXREGIONS+1], yavg[MAXREGIONS+1];
@@ -240,7 +240,7 @@ main(argc, argv)
 	FunColumnLookup(fun, NULL, bin[i], NULL,
 			&(tltyp[i]), NULL, NULL, NULL, NULL);
 	tlmin[i] = FunParamGetd(fun, "TLMIN", bin[i]+1, 0.0, &got);
-	tlmax[i] = FunParamGetd(fun, "TLMAX", bin[i]+1, 0.0, &got);
+	//	tlmax[i] = FunParamGetd(fun, "TLMAX", bin[i]+1, 0.0, &got);
 	binsiz[i] = FunParamGetd(fun, "TDBIN", bin[i]+1, 1.0, &got);
       }
     }
@@ -473,7 +473,7 @@ main(argc, argv)
 	  FunColumnLookup(fun, NULL, bin[i], NULL,
 			  &(tltyp[i]), NULL, NULL, NULL, NULL);
 	  tlmin[i] = FunParamGetd(fun, "TLMIN", bin[i]+1, 0.0, &got);
-	  tlmax[i] = FunParamGetd(fun, "TLMAX", bin[i]+1, 0.0, &got);
+	  //	  tlmax[i] = FunParamGetd(fun, "TLMAX", bin[i]+1, 0.0, &got);
 	  binsiz[i] = FunParamGetd(fun, "TDBIN", bin[i]+1, 1.0, &got);
 	}
       }
