@@ -1036,6 +1036,9 @@ main(argc, argv)
   strncpy(decstr, argv[optind+ioff++], SZ_LINE-1);
   strncpy(radstr, argv[optind+ioff++], SZ_LINE-1);
 
+  /* dolimfilt does not work with doall */
+  if( doall ) dolimfilt = 0;
+
   /* process list arguments */
   if( dolist ){
     if( !(list=OpenList(lname, rastr, decstr, radstr)) ){
@@ -1085,6 +1088,8 @@ main(argc, argv)
     if( !(adbuf=xcalloc(nrow+1, sizeof(char))) ){
       gerror(stderr, "can't allocate all (-J|-X) buffer of size %d\n", nrow);
     }
+    /* must read one event at a time, or else we can't tell the event num */
+    maxrow = 1;
   }
 
   /* activate columns specified by user, if necessary */
@@ -1371,7 +1376,7 @@ done:
     while( (ebuf = (Ev)FunTableRowGet(fun, NULL, maxrow, NULL, &got)) ){
       /* if we have a filter, then we don't know the row number til after
 	 we read the event, i.e. now */
-      if( doall & ALL_FILT ) FunInfoGet(fun, FUN_ROW, &irow, 0);
+      FunInfoGet(fun, FUN_ROW, &irow, 0);
       /* process all rows */
       for(i=0; i<got; i++){
 	/* skip rows that already were written out */
