@@ -273,7 +273,7 @@ Header (fp, dmode, name, type, n, tdim)
   int dims[SZ_LINE];
   int ndim;
   char *xfmts;
-  char *xfmt;
+  //  char *xfmt;
   char *t;
   char tbuf[SZ_LINE];
 
@@ -287,7 +287,7 @@ Header (fp, dmode, name, type, n, tdim)
   /* see values for format */
   xfmts = fmts[ind];
   xfmti = fmti[ind];
-  xfmt  = fmt[ind];
+  //  xfmt  = fmt[ind];
   /* some columns don't have a name */
   if( !name || !*name )
     name = "";
@@ -297,7 +297,7 @@ Header (fp, dmode, name, type, n, tdim)
       if( !strcasecmp(name, colfmts[i].name) ){
 	xfmts = colfmts[i].fmts;
 	xfmti = colfmts[i].fmti;
-	xfmt  = colfmts[i].fmt;
+	//	xfmt  = colfmts[i].fmt;
 	break;
       }
     }
@@ -389,8 +389,8 @@ Dashes (fp, dmode, name, type, n)
   int ind;
   int i, j;
   int xfmti;
-  char *xfmts;
-  char *xfmt;
+  //  char *xfmts;
+  //  char *xfmt;
 
   if( dmode & DMODE_MASKONLY )
     ind = NFORMAT-1;
@@ -400,9 +400,9 @@ Dashes (fp, dmode, name, type, n)
   }
 
   /* see values for format */
-  xfmts = fmts[ind];
+  //  xfmts = fmts[ind];
   xfmti = fmti[ind];
-  xfmt  = fmt[ind];
+  //  xfmt  = fmt[ind];
   /* some columns don't have a name */
   if( !name || !*name )
     name = "";
@@ -410,9 +410,9 @@ Dashes (fp, dmode, name, type, n)
   else{
     for(i=0; i<ncolfmt; i++){
       if( !strcasecmp(name, colfmts[i].name) ){
-	xfmts = colfmts[i].fmts;
+	//	xfmts = colfmts[i].fmts;
 	xfmti = colfmts[i].fmti;
-	xfmt  = colfmts[i].fmt;
+	//	xfmt  = colfmts[i].fmt;
 	break;
       }
     }
@@ -471,7 +471,7 @@ Display(fp, dmode, name, type, n, buf)
   int ival=0;
   int xn=0;
   int maskonly;
-  int xfmti;
+  //  int xfmti;
   char *xfmts;
   char *xfmt;
   char tbuf[SZ_LINE];
@@ -494,7 +494,7 @@ Display(fp, dmode, name, type, n, buf)
 
   /* see values for format */
   xfmts = fmts[ind];
-  xfmti = fmti[ind];
+  //  xfmti = fmti[ind];
   xfmt  = fmt[ind];
   /* some columns don't have a name */
   if( !name )
@@ -504,7 +504,7 @@ Display(fp, dmode, name, type, n, buf)
     for(i=0; i<ncolfmt; i++){
       if( !strcasecmp(name, colfmts[i].name) ){
 	xfmts = colfmts[i].fmts;
-	xfmti = colfmts[i].fmti;
+	//	xfmti = colfmts[i].fmti;
 	xfmt  = colfmts[i].fmt;
 	break;
       }
