@@ -85,8 +85,9 @@ int _FunSpecialFile(fname, type, name, tail, special, mlen)
   if( ((s=strchr(fname, '['))||(s=strchr(fname, ',')))    	&& 
       !strncasecmp(s+1, tbuf, len) && (t=strchr(s, ')'))  	&&
       ((*(t+1) == ')') || (*(t+1) == ',') || (*(t+1) == ']'))	){
-    strncpy(name, fname, MIN(mlen,s-fname));
-    name[mlen-1] = '\0';
+    tlen = MIN(mlen-1,s-fname);
+    strncpy(name, fname, tlen);
+    name[tlen] = '\0';
     while( *(s+len+1) == '(' ) s++;
     tlen = MIN(mlen-1,t-(s+len+1));
     strncpy(special, s+len+1, tlen);
@@ -774,7 +775,7 @@ off_t _FunImageSkip(fun, tail)
   /* we look for a plane specification for each axis > 2 */
   for(i=3; i<=naxes; i++){
     plane = strtol(s, &t, 10);
-    if( (s == t) )
+    if( s == t )
       plane = 1;
     else if( (plane < 1) || (plane > ft_naxis(fun->header, i)) )
       return -1;
