@@ -273,6 +273,7 @@ static int _FunTextGetLine(fun, iline, lbuf, llen)
 #endif
 {
   int len;
+  unsigned int i;
   char *lptr=NULL;
 
   /* initialize */
@@ -303,6 +304,12 @@ static int _FunTextGetLine(fun, iline, lbuf, llen)
       return 0;
     }
   }
+  /* make sure all characters are valid */
+  for(i=0; i<strlen(lbuf); i++){
+    if( !isprint(lbuf[i]) && !isspace(lbuf[i]) ){
+      return 0;
+    }
+  }
   return strlen(lbuf);
 }
 
@@ -932,7 +939,8 @@ header:
 	if( fun->parsers[p]->state & PARSE_STATE_BAD ) continue;
 	if( fun->parsers[p]->types[0] == PARSE_STRING ){
 	  for(t=0; t<fun->parsers[p]->ntoken; t++){
-	    if( !strchr(fun->parsers[p]->tokens[t].sval, ':') ){
+	    if( !fun->parsers[p]->tokens[t].sval ||
+		!strchr(fun->parsers[p]->tokens[t].sval, ':') ){
 	      nheader++;
 	      break;
 	    }
@@ -1090,7 +1098,6 @@ headguess:
 	case PARSE_DASH:
 	case PARSE_COMMENT:
 	case PARSE_EOL:
-	  break;
 	default:
 	  gwarning(stderr,
 	   "internal text parser error: invalid type in text header: '%c'\n", 
