@@ -1,8 +1,8 @@
 /*** File libwcs/fitsfile.c
- *** November 2, 2006
- *** By Doug Mink, dmink@cfa.harvard.edu
+ *** June 24, 2016
+ *** By Jessica Mink, jmink@cfa.harvard.edu
  *** Harvard-Smithsonian Center for Astrophysics
- *** Copyright (C) 1996-2006
+ *** Copyright (C) 1996-2016
  *** Smithsonian Astrophysical Observatory, Cambridge, MA, USA
 
     This library is free software; you can redistribute it and/or
@@ -20,8 +20,8 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Correspondence concerning WCSTools should be addressed as follows:
-           Internet email: dmink@cfa.harvard.edu
-           Postal address: Doug Mink
+           Internet email: jmink@cfa.harvard.edu
+           Postal address: Jessica Mink
                            Smithsonian Astrophysical Observatory
                            60 Garden St.
                            Cambridge, MA 02138 USA
@@ -87,15 +87,15 @@
 
 static int verbose=0;		/* Print diagnostics */
 static char fitserrmsg[80];
-static int fitsinherit = 1;		/* Append primary header to extension header */
+static int fitsinherit = 1;	/* Append primary header to extension header */
 void
 setfitsinherit (inh)
 int inh;
 {fitsinherit = inh; return;}
 
-static int ibhead = 0;		/* Number of bytes read before header starts */
+static off_t ibhead = 0;	/* Number of bytes read before header starts */
 
-int
+off_t
 getfitsskip()
 {return (ibhead);}
 
@@ -134,6 +134,7 @@ int	*nbhead;	/* Number of bytes before start of data (returned) */
     char *pheadend;
     int inherit = 1;	/* Value of INHERIT keyword in FITS extension header */
     int extfound = 0;	/* Set to one if desired FITS extension is found */
+    int npcount;
 
     pheader = NULL;
     lprim = 0;
@@ -173,7 +174,7 @@ int	*nbhead;	/* Number of bytes before start of data (returned) */
 #endif
 
     if (ext != NULL) {
-	if (isnum (ext+1))
+	if (isnum (ext+1) == 1)
 	    extnum = atoi (ext+1);
 	else {
 	    extnum = -2;
@@ -210,11 +211,13 @@ int	*nbhead;	/* Number of bytes before start of data (returned) */
     /* Read FITS header from input file one FITS block at a time */
     irec = 0;
     ibhead = 0;
-    while (irec < 100) {
+    while (irec < 500) {
 	nbytes = FITSBLOCK;
 	for (ntry = 0; ntry < 10; ntry++) {
 	    for (i = 0; i < 2884; i++) fitsbuf[i] = 0;
 	    nbr = read (fd, fitsbuf, nbytes);
+	    if (verbose)
+		fprintf (stderr,"FITSRHEAD: %d header bytes read\n",nbr);
 
 	    /* Short records allowed only if they have the last header line */
 	    if (nbr < nbytes) {
@@ -233,16 +236,37 @@ int	*nbhead;	/* Number of bytes before start of data (returned) */
 #endif
 			    (void)close (fd);
 			free (header);
-			if (pheader != NULL)
-			    return (pheader);
+			/* if (pheader != NULL)
+			    return (pheader); */
     			if (extnum != -1 && !extfound) {
-			    if (extnum < 0)
-	    			fprintf (stderr, "FITSRHEAD: Extension %s not found in file %s\n",extnam, filename);
-			    else
-	    			fprintf (stderr, "FITSRHEAD: Extension %d not found in file %s\n",extnum, filename);
+			    *ext = (char) 0;
+			    if (extnum < 0) {
+	    			snprintf (fitserrmsg,79,
+				    "FITSRHEAD: Extension %s not found in file %s",
+				    extnam, filename);
+				}
+			    else {
+	    			snprintf (fitserrmsg,79,
+				    "FITSRHEAD: Extension %d not found in file %s",
+				    extnum, filename);
+				}
+			    *ext = cext;
+			    }
+			else if (hdu > 0) {
+	    		    snprintf (fitserrmsg,79,
+				"FITSRHEAD: No extensions found in file %s", filename);
+			    hdu = 0;
+			    if (pheader != NULL) {
+				*lhead = nbprim;
+				*nbhead = nbprim;
+				return (pheader);
+				}
+			    break;
+			    }
+			else {
+	    		    snprintf (fitserrmsg,79,
+				"FITSRHEAD: No header found in file %s", filename);
 			    }
-			else
-	    		    fprintf (stderr, "FITSRHEAD: No header found in file %s\n", filename);
 			return (NULL);
 			}
 		    }
@@ -253,32 +277,46 @@ int	*nbhead;	/* Number of bytes before start of data (returned) */
 		break;
 	    }
 
-	/* Move current FITS record into header string */
+	/* Replace control characters and nulls with spaces */
 	for (i = 0; i < 2880; i++)
-	    if (fitsbuf[i] < 32) fitsbuf[i] = 32;
+	    if (fitsbuf[i] < 32 || i > nbr) fitsbuf[i] = 32;
+	if (nbr < 2880)
+	    nbr = 2880;
+
+	/* Move current FITS record into header string */
 	strncpy (headnext, fitsbuf, nbr);
 	*nbhead = *nbhead + nbr;
 	nrec = nrec + 1;
-	*(headnext+nbr) = 0;
+	*(headnext+nbr+1) = 0;
 	ibhead = ibhead + 2880;
+	if (verbose)
+	    fprintf (stderr,"FITSRHEAD: %ld bytes in header\n",ibhead);
 
 	/* Check to see if this is the final record in this header */
 	headend = ksearch (fitsbuf,"END");
 	if (headend == NULL) {
 
-	    /* Increase size of header buffer by 4 blocks if too small */
+	    /* Double size of header buffer if too small */
 	    if (nrec * FITSBLOCK > nbh) {
-		nbh0 = nbh;
-		nbh = (nrec + 4) * FITSBLOCK + 4;
+		nbh0 = nbh - 4;
+		nbh = (nrec * 2 * FITSBLOCK) + 4;
 		newhead = (char *) calloc (1,(unsigned int) nbh);
-		for (i = 0; i < nbh0; i++)
-		    newhead[i] = header[i];
-		free (header);
-		header = newhead;
-		(void) hlength (header, nbh);
-		headnext = header + *nbhead - FITSBLOCK;
+		if (newhead) {
+		    for (i = 0; i < nbh0; i++)
+			newhead[i] = header[i];
+		    free (header);
+		    newhead[nbh-3] = (char) 0;
+		    header = newhead;
+		    (void) hlength (header, nbh);
+		    headnext = header + ((nrec-1) * FITSBLOCK);
+		    }
+		else {
+		    fprintf (stderr,"FITSRHEAD: %d bytes cannot be allocated for header\n",nbh);
+		    exit (1);
+		    }
 		}
-	    headnext = headnext + FITSBLOCK;
+	    else
+		headnext = headnext + FITSBLOCK;
 	    }
 
 	else {
@@ -290,10 +328,11 @@ int	*nbhead;	/* Number of bytes before start of data (returned) */
 		nbprim = nrec * FITSBLOCK;
 		headend = ksearch (header,"END");
 		lprim = headend + 80 - header;
-		pheader = (char *) calloc ((unsigned int) nbprim, 1);
+		pheader = (char *) calloc ((unsigned int) (nbprim + 1), 1);
 		for (i = 0; i < lprim; i++)
 		    pheader[i] = header[i];
-		strncpy (pheader, header, lprim);
+		for (i = lprim; i < nbprim; i++)
+		    pheader[i] = ' ';
 		}
 
 	    /* If header has no data, start with the next record */
@@ -304,11 +343,12 @@ int	*nbhead;	/* Number of bytes before start of data (returned) */
 		    headnext = header;
 		    *headend = ' ';
 		    headend = NULL;
-		    /* nrec = 1; */
+		    nrec = 1;
 		    hdu = hdu + 1;
 		    }
-		else
+		else {
 		    break;
+		    }
 		}
 
 	    /* If this is the desired header data unit, keep it */
@@ -333,28 +373,43 @@ int	*nbhead;	/* Number of bytes before start of data (returned) */
 		if (naxis > 0) {
 		    ibpix = 0;
 		    hgeti4 (header,"BITPIX",&ibpix);
-		    if (ibpix < 0)
+		    if (ibpix < 0) {
 			nbpix = -ibpix / 8;
-		    else
+			}
+		    else {
 			nbpix = ibpix / 8;
+			}
 		    nax1 = 1;
 		    hgeti4 (header,"NAXIS1",&nax1);
 		    nax2 = 1;
-		    if (naxis > 1)
+		    if (naxis > 1) {
 			hgeti4 (header,"NAXIS2",&nax2);
+			}
 		    nax3 = 1;
-		    if (naxis > 2)
+		    if (naxis > 2) {
 			hgeti4 (header,"NAXIS3",&nax3);
+			}
 		    nax4 = 1;
-		    if (naxis > 3)
+		    if (naxis > 3) {
 			hgeti4 (header,"NAXIS4",&nax4);
+			}
 		    nbskip = nax1 * nax2 * nax3 * nax4 * nbpix;
 		    nblock = nbskip / 2880;
-		    if (nblock*2880 < nbskip)
+		    if (nblock*2880 < nbskip) {
 			nblock = nblock + 1;
+			}
+		    npcount = 0;
+		    hgeti4 (header,"PCOUNT", &npcount);
+		    if (npcount > 0) {
+			nbskip = nbskip + npcount;
+			nblock = nbskip / 2880;
+			if (nblock*2880 < nbskip)
+			    nblock = nblock + 1;
+			}
 		    }
-		else
+		else {
 		    nblock = 0;
+		    }
 		*nbhead = *nbhead + (nblock * 2880);
 
 		/* Set file pointer to beginning of next header/data unit */
@@ -376,14 +431,16 @@ int	*nbhead;	/* Number of bytes before start of data (returned) */
 				ipos = ipos + nbr;
 				break;
 				}
-			    else
+			    else {
 				ipos = ipos + nbytes;
+				}
 			    }
 			npos = nblock * 2880;
 			}
 		    if (ipos < npos) {
 			snprintf (fitserrmsg,79,"FITSRHEAD: %d / %d bytes skipped\n",
 				 ipos,npos);
+			extfound = 0;
 			break;
 			}
 		    }
@@ -391,8 +448,9 @@ int	*nbhead;	/* Number of bytes before start of data (returned) */
 		headend = NULL;
 		nrec = 1;
 		}
-	    else
+	    else {
 		break;
+		}
 	    }
 	}
 
@@ -407,8 +465,10 @@ int	*nbhead;	/* Number of bytes before start of data (returned) */
 	    fprintf (stderr, "FITSRHEAD: Extension %s not found in file %s\n",extnam, filename);
 	else
 	    fprintf (stderr, "FITSRHEAD: Extension %d not found in file %s\n",extnum, filename);
-	if (pheader != NULL)
+	if (pheader != NULL) {
 	    free (pheader);
+	    pheader = NULL;
+	    }
 	return (NULL);
 	}
 
@@ -432,16 +492,13 @@ int	*nbhead;	/* Number of bytes before start of data (returned) */
 	}
 
     /* Append primary data header to extension header */
-    if (pheader != NULL && extnum != 0 && fitsinherit) {
+    if (pheader != NULL && extnum != 0 && fitsinherit && hdu > 0) {
 	extname[0] = 0;
 	hgets (header, "XTENSION", 32, extname);
 	if (!strcmp (extname,"IMAGE")) {
 	    strncpy (header, "SIMPLE  ", 8);
 	    hputl (header, "SIMPLE", 1);
 	    }
-	hputs (header,"COMMENT","-------------------------------------------");
-	hputs (header,"COMMENT","Information from Primary Header");
-	hputs (header,"COMMENT","-------------------------------------------");
 	headend = blsearch (header,"END");
 	if (headend == NULL)
 	    headend = ksearch (header, "END");
@@ -455,7 +512,7 @@ int	*nbhead;	/* Number of bytes before start of data (returned) */
 	hdel (pheader, "EXTEND");
 	hputl (pheader, "ROOTEND",1);
 	pheadend = ksearch (pheader,"END");
-	lprim = pheadend + 80 - pheader;
+	lprim = pheadend + 320 - pheader;
 	if (lext + lprim > nbh) {
 	    nrec = (lext + lprim) / FITSBLOCK;
 	    if (FITSBLOCK*nrec < lext+lprim)
@@ -469,9 +526,18 @@ int	*nbhead;	/* Number of bytes before start of data (returned) */
 	    headend = header + lext;
 	    (void) hlength (header, *lhead);
 	    }
+	hputs (header,"COMMENT","-------------------------------------------");
+	hputs (header,"COMMENT","Information from Primary Header");
+	hputs (header,"COMMENT","-------------------------------------------");
+	headend = blsearch (header,"END");
+	if (headend == NULL)
+	    headend = ksearch (header, "END");
 	pheader[lprim] = 0;
 	strncpy (headend, pheader, lprim);
-	free (pheader);
+	if (pheader != NULL) {
+	    free (pheader);
+	    pheader = NULL;
+	    }
 	}
 
     ibhead = *nbhead - ibhead;
@@ -494,7 +560,8 @@ int	*nbhead;	/* Number of bytes before start of data (returned) */
     int fd;
     char *header;	/* FITS image header (filled) */
     int nbytes, i, ndiff;
-    int nbr, irec, offset;
+    int nbr, irec;
+    off_t offset;
     char *mwcs;		/* Pointer to WCS name separated by % */
     char *headstart;
     char *newhead;
@@ -590,7 +657,8 @@ int	nlog;		/* Note progress mod this rows */
     int fd;		/* File descriptor */
     int nbimage, naxis1, naxis2, bytepix, nbread;
     int bitpix, naxis, nblocks, nbytes, nbr;
-    int x1, y1, nbline, impos, nblin, nyleft;
+    int x1, y1, nbline, nyleft;
+    off_t impos, nblin;
     char *image, *imline, *imlast;
     int ilog = 0;
     int row;
@@ -852,14 +920,14 @@ char	*header;	/* FITS header for image (previously read) */
 
 	fd = fitsropen (filename);
 	if (fd < 0) {
-	    snprintf (fitserrmsg,79, "FITSRIMAGE:  cannot read file %s\n", filename);
+	    snprintf (fitserrmsg,79, "FITSRFULL:  cannot read file %s\n", filename);
 	    return (NULL);
 	    }
 
 	/* Skip over FITS header and whatever else needs to be skipped */
 	if (lseek (fd, nbhead, SEEK_SET) < 0) {
 	    (void)close (fd);
-	    snprintf (fitserrmsg,79, "FITSRIMAGE:  cannot skip header of file %s\n",
+	    snprintf (fitserrmsg,79, "FITSRFULL:  cannot skip header of file %s\n",
 		     filename);
 	    return (NULL);
 	    }
@@ -874,7 +942,7 @@ char	*header;	/* FITS header for image (previously read) */
     if (!simple) {
 	nbytes = getfilesize (filename) - nbhead;
 	if ((image = (char *) malloc (nbytes + 1)) == NULL) {
-	    /* snprintf (fitserrmsg,79, "FITSRIMAGE:  %d-byte image buffer cannot be allocated\n"); */
+	    snprintf (fitserrmsg,79, "FITSRFULL:  %d-byte image buffer cannot be allocated\n",nbytes+1);
 	    (void)close (fd);
 	    return (NULL);
 	    }
@@ -887,7 +955,7 @@ char	*header;	/* FITS header for image (previously read) */
     bitpix = 0;
     hgeti4 (header,"BITPIX",&bitpix);
     if (bitpix == 0) {
-	/* snprintf (fitserrmsg,79, "FITSRIMAGE:  BITPIX is 0; image not read\n"); */
+	snprintf (fitserrmsg,79, "FITSRFULL:  BITPIX is 0; image not read\n");
 	(void)close (fd);
 	return (NULL);
 	}
@@ -933,7 +1001,7 @@ char	*header;	/* FITS header for image (previously read) */
 	(void)close (fd);
 #endif
     if (nbr < nbimage) {
-	snprintf (fitserrmsg,79, "FITSRIMAGE:  %d of %d bytes read from file %s\n",
+	snprintf (fitserrmsg,79, "FITSRFULL:  %d of %d image bytes read from file %s\n",
 		 nbr, nbimage, filename);
 	return (NULL);
 	}
@@ -1040,16 +1108,18 @@ int	*nbhead;	/* Number of characters before table starts */
 /* Make sure this file is really a FITS table file */
     temp[0] = 0;
     (void) hgets (header,"XTENSION",16,temp);
-    if (strncmp (temp, "TABLE", 5)) {
-	snprintf (fitserrmsg,79, "FITSRTOPEN:  %s is not a FITS table file\n",inpath);
+    if (strlen (temp) == 0) {
+	snprintf (fitserrmsg,79,
+		  "FITSRTOPEN:  %s is not a FITS table file\n",inpath);
 	free ((void *) header);
 	return (0);
 	}
 
 /* If it is a FITS file, get table information from the header */
-    else {
+    else if (!strcmp (temp, "TABLE") || !strcmp (temp, "BINTABLE")) {
 	if (fitsrthead (header, nk, kw, nrows, nchar)) {
-	    snprintf (fitserrmsg,79, "FITSRTOPEN: Cannot read FITS table from %s\n",inpath);
+	    snprintf (fitserrmsg,79,
+		      "FITSRTOPEN: Cannot read FITS table from %s\n",inpath);
 	    free ((void *) header);
 	    return (-1);
 	    }
@@ -1061,6 +1131,15 @@ int	*nbhead;	/* Number of characters before table starts */
 	    return (fd);
 	    }
 	}
+
+/* If it is another FITS extension note it and return */
+    else {
+	snprintf (fitserrmsg,79,
+		  "FITSRTOPEN:  %s is a %s extension, not table\n",
+		  inpath, temp);
+	free ((void *) header);
+	return (0);
+	}
 }
 
 static struct Keyword *pw;	/* Structure for all entries */
@@ -1081,21 +1160,28 @@ int	*nchar;		/* Number of characters in one table row (returned) */
 {
     struct Keyword *rw;	/* Structure for desired entries */
     int nfields;
-    int ifield,ik,ln, i;
+    int ifield, ik, i, ikf, ltform, kl;
     char *h0, *h1, *tf1, *tf2;
     char tname[12];
     char temp[16];
     char tform[16];
     int tverb;
+    int bintable = 0;
 
     h0 = header;
 
 /* Make sure this is really a FITS table file header */
     temp[0] = 0;
     hgets (header,"XTENSION",16,temp);
-    if (strncmp (temp, "TABLE", 5) != 0) {
-	snprintf (fitserrmsg,79, "FITSRTHEAD:  Not a FITS table file\n");
-	free (temp);
+    if (strlen (temp) == 0) {
+	snprintf (fitserrmsg,79, "FITSRTHEAD:  Not a FITS table header\n");
+	return (-1);
+	}
+    else if (!strcmp (temp, "BINTABLE")) {
+	bintable = 1;
+	}
+    else if (strcmp (temp, "TABLE")) {
+	snprintf (fitserrmsg,79, "FITSRTHEAD:  %s extension, not TABLE\n",temp);
 	return (-1);
 	}
 
@@ -1135,35 +1221,65 @@ int	*nchar;		/* Number of characters in one table row (returned) */
 
     tverb = verbose;
     verbose = 0;
+    ikf = 0;
 
     for (ifield = 0; ifield < nfields; ifield++) {
 
-    /* First column of field */
+    /* Name of field */
 	for (i = 0; i < 12; i++) tname[i] = 0;
-	sprintf (tname, "TBCOL%d", ifield+1);
+	sprintf (tname, "TTYPE%d", ifield+1);;
+	temp[0] = 0;
 	h1 = ksearch (h0,tname);
-	pw[ifield].kf = 0;
-	hgeti4 (h0,tname, &pw[ifield].kf);
+	h0 = h1;
+	hgets (h0,tname,16,temp);
+	strcpy (pw[ifield].kname,temp);
+	pw[ifield].lname = strlen (pw[ifield].kname);
+
+    /* Sequence of field on line */
+	pw[ifield].kn = ifield + 1;
+
+    /* First column of field */
+	if (bintable)
+	    pw[ifield].kf = ikf;
+	else {
+	    for (i = 0; i < 12; i++) tname[i] = 0;
+	    sprintf (tname, "TBCOL%d", ifield+1);
+	    pw[ifield].kf = 0;
+	    hgeti4 (h0,tname, &pw[ifield].kf);
+	    }
 
     /* Length of field */
 	for (i = 0; i < 12; i++) tname[i] = 0;
 	sprintf (tname, "TFORM%d", ifield+1);;
 	tform[0] = 0;
 	hgets (h0,tname,16,tform);
-	tf1 = tform + 1;
-	tf2 = strchr (tform,'.');
-	if (tf2 != NULL)
-	    *tf2 = ' ';
-	pw[ifield].kl = atoi (tf1);
-
-    /* Name of field */
-	for (i = 0; i < 12; i++) tname[i] = 0;
-	sprintf (tname, "TTYPE%d", ifield+1);;
-	temp[0] = 0;
-	hgets (h0,tname,16,temp);
-	strcpy (pw[ifield].kname,temp);
-	lpnam[ifield] = strlen (pw[ifield].kname);
-	h0 = h1;
+	strcpy (pw[ifield].kform, tform);
+	ltform = strlen (tform);
+	if (tform[ltform-1] == 'A') {
+	    pw[ifield].kform[0] = 'A';
+	    for (i = 0; i < ltform-1; i++)
+		pw[ifield].kform[i+1] = tform[i];
+	    pw[ifield].kform[ltform] = (char) 0;
+	    tf1 = pw[ifield].kform + 1;
+	    kl = atof (tf1);
+	    }
+	else if (!strcmp (tform,"I"))
+	    kl = 2;
+	else if (!strcmp (tform, "J"))
+	    kl = 4;
+	else if (!strcmp (tform, "E"))
+	    kl = 4;
+	else if (!strcmp (tform, "D"))
+	    kl = 8;
+	else {
+	    tf1 = tform + 1;
+	    tf2 = strchr (tform,'.');
+	    if (tf2 != NULL)
+		*tf2 = ' ';
+	    kl = atoi (tf1);
+	    }
+	pw[ifield].kl = kl;
+	ikf = ikf + kl;
 	}
 
 /* Set up table for access to desired fields */
@@ -1184,10 +1300,9 @@ int	*nchar;		/* Number of characters in one table row (returned) */
     for (ik = 0; ik < *nk; ik++) {
 	if (rw[ik].kn <= 0) {
 	    for (ifield = 0; ifield < nfields; ifield++) {
-		ln = lpnam[ifield];
-		if (rw[ik].lname > ln)
-		    ln = rw[ik].lname;
-		if (strncmp (pw[ifield].kname, rw[ik].kname, ln) == 0) {
+		if (rw[ik].lname != pw[ifield].lname)
+		    continue;
+		if (strcmp (pw[ifield].kname, rw[ik].kname) == 0) {
 		    break;
 		    }
 		}
@@ -1199,6 +1314,7 @@ int	*nchar;		/* Number of characters in one table row (returned) */
 	rw[ik].kn = ifield + 1;
 	rw[ik].kf = pw[ifield].kf - 1;
 	rw[ik].kl = pw[ifield].kl;
+	strcpy (rw[ik].kform, pw[ifield].kform);
 	strcpy (rw[ik].kname, pw[ifield].kname);
 	}
 
@@ -1280,9 +1396,30 @@ char	*entry;		/* Row or entry from table */
 struct Keyword *kw;	/* Table column information from FITS header */
 {
     char temp[30];
+    short i;
+    int j;
+    float r;
+    double d;
 
-    if (ftgetc (entry, kw, temp, 30))
-	return ( (short) atof (temp) );
+    if (ftgetc (entry, kw, temp, 30)) {
+	if (!strcmp (kw->kform, "I"))
+	    moveb (temp, (char *) &i, 2, 0, 0);
+	else if (!strcmp (kw->kform, "J")) {
+	    moveb (temp, (char *) &j, 4, 0, 0);
+	    i = (short) j;
+	    }
+	else if (!strcmp (kw->kform, "E")) {
+	    moveb (temp, (char *) &r, 4, 0, 0);
+	    i = (short) r;
+	    }
+	else if (!strcmp (kw->kform, "D")) {
+	    moveb (temp, (char *) &d, 8, 0, 0);
+	    i = (short) d;
+	    }
+	else
+	    i = (short) atof (temp);
+	return (i);
+	}
     else
 	return ((short) 0);
 }
@@ -1297,9 +1434,30 @@ char	*entry;		/* Row or entry from table */
 struct Keyword *kw;	/* Table column information from FITS header */
 {
     char temp[30];
+    short i;
+    int j;
+    float r;
+    double d;
 
-    if (ftgetc (entry, kw, temp, 30))
-	return ( (int) atof (temp) );
+    if (ftgetc (entry, kw, temp, 30)) {
+	if (!strcmp (kw->kform, "I")) {
+	    moveb (temp, (char *) &i, 2, 0, 0);
+	    j = (int) i;
+	    }
+	else if (!strcmp (kw->kform, "J"))
+	    moveb (temp, (char *) &j, 4, 0, 0);
+	else if (!strcmp (kw->kform, "E")) {
+	    moveb (temp, (char *) &r, 4, 0, 0);
+	    j = (int) r;
+	    }
+	else if (!strcmp (kw->kform, "D")) {
+	    moveb (temp, (char *) &d, 8, 0, 0);
+	    j = (int) d;
+	    }
+	else
+	    j = (int) atof (temp);
+	return (j);
+	}
     else
 	return (0);
 }
@@ -1314,9 +1472,30 @@ char	*entry;		/* Row or entry from table */
 struct Keyword *kw;	/* Table column information from FITS header */
 {
     char temp[30];
+    short i;
+    int j;
+    float r;
+    double d;
 
-    if (ftgetc (entry, kw, temp, 30))
-	return ( (float) atof (temp) );
+    if (ftgetc (entry, kw, temp, 30)) {
+	if (!strcmp (kw->kform, "I")) {
+	    moveb (temp, (char *) &i, 2, 0, 0);
+	    r = (float) i;
+	    }
+	else if (!strcmp (kw->kform, "J")) {
+	    moveb (temp, (char *) &j, 4, 0, 0);
+	    r = (float) j;
+	    }
+	else if (!strcmp (kw->kform, "E"))
+	    moveb (temp, (char *) &r, 4, 0, 0);
+	else if (!strcmp (kw->kform, "D")) {
+	    moveb (temp, (char *) &d, 8, 0, 0);
+	    r = (float) d;
+	    }
+	else
+	    r = (float) atof (temp);
+	return (r);
+	}
     else
 	return ((float) 0.0);
 }
@@ -1331,9 +1510,30 @@ char	*entry;		/* Row or entry from table */
 struct Keyword *kw;	/* Table column information from FITS header */
 {
     char temp[30];
+    short i;
+    int j;
+    float r;
+    double d;
 
-    if (ftgetc (entry, kw, temp, 30))
-	return ( atof (temp) );
+    if (ftgetc (entry, kw, temp, 30)) {
+	if (!strcmp (kw->kform, "I")) {
+	    moveb (temp, (char *) &i, 2, 0, 0);
+	    d = (double) i;
+	    }
+	else if (!strcmp (kw->kform, "J")) {
+	    moveb (temp, (char *) &j, 4, 0, 0);
+	    d = (double) j;
+	    }
+	else if (!strcmp (kw->kform, "E")) {
+	    moveb (temp, (char *) &r, 4, 0, 0);
+	    d = (double) r;
+	    }
+	else if (!strcmp (kw->kform, "D"))
+	    moveb (temp, (char *) &d, 8, 0, 0);
+	else
+	    d = atof (temp);
+	return (d);
+	}
     else
 	return ((double) 0.0);
 }
@@ -1796,7 +1996,7 @@ char	*header;	/* FITS image header */
 	*(endhead++) = ' ';
     
     nbw = write (fd, header, nbytes);
-    if (nbw < nbhead) {
+    if (nbw < nbytes) {
 	fprintf (stderr, "FITSWHEAD:  wrote %d / %d bytes of header to file %s\n",
 		 nbw, nbytes, filename);
 	(void)close (fd);
@@ -1825,7 +2025,7 @@ char	*header;	/* FITS image header */
     fitsinherit = 0;
     oldheader = fitsrhead (filename, &lhead, &nbhead);
     if (oldheader == NULL) {
-	fprintf (stderr, "FITSWEXHEAD:  file %s cannot be read\n", filename);
+	snprintf (fitserrmsg, 79, "FITSWEXHEAD:  file %s cannot be read\n", filename);
 	return (-1);
 	}
     nbold = fitsheadsize (oldheader);
@@ -1833,7 +2033,7 @@ char	*header;	/* FITS image header */
 
     /* Return if the new header is bigger than the old header */
     if (nbnew > nbold) {
-	fprintf (stderr, "FITSWEXHEAD:  old header %d bytes, new header %d bytes\n", nbold,nbnew);
+	snprintf (fitserrmsg, 79, "FITSWEXHEAD:  old header %d bytes, new header %d bytes\n", nbold,nbnew);
 	free (oldheader);
 	oldheader = NULL;
 	return (-1);
@@ -1872,7 +2072,7 @@ char	*header;	/* FITS image header */
     if (ext != NULL)
 	*ext = cext;
     if (fd < 3) {
-	fprintf (stderr, "FITSWEXHEAD:  file %s not writeable\n", filename);
+	snprintf (fitserrmsg, 79, "FITSWEXHEAD:  file %s not writeable\n", filename);
 	return (-1);
 	}
 
@@ -1884,7 +2084,7 @@ char	*header;	/* FITS image header */
     (void)close (fd);
     free (oldheader);
     oldheader = NULL;
-    if (nbw < nbhead) {
+    if (nbw < nbold) {
 	fprintf (stderr, "FITSWHEAD:  wrote %d / %d bytes of header to file %s\n",
 		 nbw, nbold, filename);
 	return (-1);
@@ -1901,28 +2101,36 @@ char    *filename;      /* Name of file for which to find size */
 {
     int diskfile;
     char keyword[16];
+    char *comma;
     int nbr;
 
     /* First check to see if this is an assignment */
     if (strchr (filename, '='))
 	return (0);
 
-    /* Then check file extension */
-    else if (strsrch (filename, ".fit") ||
-	strsrch (filename, ".fits") ||
-	strsrch (filename, ".fts"))
-	return (1);
-
     /* Check for stdin (input from pipe) */
     else if (!strcasecmp (filename,"stdin"))
 	return (1);
 
+    /* Then check file extension
+    else if (strsrch (filename, ".fit") ||
+	strsrch (filename, ".fits") ||
+	strsrch (filename, ".fts"))
+	return (1); */
+
     /* If no FITS file extension, try opening the file */
     else {
-	if ((diskfile = open (filename, O_RDONLY)) < 0)
+	if ((comma = strchr (filename,',')))
+	    *comma = (char) 0;
+	if ((diskfile = open (filename, O_RDONLY)) < 0) {
+	    if (comma)
+		*comma = ',';
 	    return (0);
+	    }
 	else {
 	    nbr = read (diskfile, keyword, 8);
+	    if (comma)
+		*comma = ',';
 	    close (diskfile);
 	    if (nbr < 8)
 		return (0);
@@ -1934,6 +2142,7 @@ char    *filename;      /* Name of file for which to find size */
 	}
 }
 
+
 /* FITSHEADSIZE -- Find size of FITS header */
 
 int
@@ -1952,12 +2161,33 @@ char	*header;	/* FITS header */
     return (nblocks * FITSBLOCK);
 }
 
+
 /* Print error message */
 void
 fitserr ()
 {   fprintf (stderr, "%s\n",fitserrmsg);
     return; }
 
+
+/* MOVEB -- Copy nbytes bytes from source+offs to dest+offd (any data type) */
+
+void
+moveb (source, dest, nbytes, offs, offd)
+
+char *source;	/* Pointer to source */
+char *dest;	/* Pointer to destination */
+int nbytes;	/* Number of bytes to move */
+int offs;	/* Offset in bytes in source from which to start copying */
+int offd;	/* Offset in bytes in destination to which to start copying */
+{
+char *from, *last, *to;
+        from = source + offs;
+        to = dest + offd;
+        last = from + nbytes;
+        while (from < last) *(to++) = *(from++);
+        return;
+}
+
 /*
  * Feb  8 1996	New subroutines
  * Apr 10 1996	Add subroutine list at start of file
@@ -2070,4 +2300,29 @@ fitserr ()
  * Nov  2 2006	Change all realloc() calls to calloc()
  *
  * Jan  5 2007	In fitsrtail(), change control characters in header to spaces
+ * Apr 30 2007	Improve error reporting in FITSRFULL
+ * Nov 28 2007	Add support to BINTABLE in ftget*() and fitsrthead()
+ * Dec 20 2007	Add data heap numerated by PCOUNT when skipping HDU in fitsrhead()
+ * Dec 20 2007	Return NULL pointer if fitsrhead() cannot find requested HDU
+ *
+ * Apr  7 2008	Drop comma from name when reading file in isfits()
+ * Jun 27 2008	Do not append primary data header if it is the only header
+ * Nov 21 2008	In fitswhead(), print message if too few bytes written
+ *
+ * Sep 18 2009	In fitswexhead() write to error string instead of stderr
+ * Sep 22 2009	In fitsrthead(), fix lengths for ASCII numeric table entries
+ * Sep 25 2009	Add subroutine moveb() and fix calls to it
+ * Sep 25 2009	Fix several small errors found by Jessicalas Burke
+ *
+ * Mar 29 2010	In fitswhead(), always pad blocks to 2880 bytes with spaces
+ * Mar 31 2010	In fitsrhead(), fix bug reading long primary headers
+ *
+ * Sep 15 2011	In fitsrsect() declare impos and nblin off_t
+ * Sep 15 2011	In fitsrtail() declare offset off_t
+ * Sep 15 2011	Declare global variable ibhead off_t
+ *
+ * Jul 25 2014	Fix bug when reallocating buffer for long headers
+ *
+ * Jun  9 2016	Fix isnum() tests for added coloned times and dashed dates
+ * Jun 24 2016	Add 1 to allocation of pheader for trailing null, fix by Ole Streicher
  */
