@@ -1,9 +1,9 @@
 /*** File wcslib/tnxpos.c
- *** January 8, 2007
- *** By Doug Mink, dmink@cfa.harvard.edu
+ *** September 17, 2008
+ *** By Jessica Mink, jmink@cfa.harvard.edu
  *** Harvard-Smithsonian Center for Astrophysics
  *** After IRAF mwcs/wftnx.x and mwcs/wfgsurfit.x
- *** Copyright (C) 1998-2007
+ *** Copyright (C) 1998-2008
  *** Smithsonian Astrophysical Observatory, Cambridge, MA, USA
 
     This library is free software; you can redistribute it and/or
@@ -21,8 +21,8 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Correspondence concerning WCSTools should be addressed as follows:
-           Internet email: dmink@cfa.harvard.edu
-           Postal address: Doug Mink
+           Internet email: jmink@cfa.harvard.edu
+           Postal address: Jessica Mink
                            Smithsonian Astrophysical Observatory
                            60 Garden St.
                            Cambridge, MA 02138 USA
@@ -72,8 +72,8 @@ static void wf_gsb1cheb();
 int
 tnxinit (header, wcs)
 
-char	*header;		/* FITS header */
-struct WorldCoor *wcs;		/* pointer to WCS structure */
+const char *header;	/* FITS header */
+struct WorldCoor *wcs;	/* pointer to WCS structure */
 {
     struct IRAFsurface *wf_gsopen();
     char *str1, *str2, *lngstr, *latstr;
@@ -165,7 +165,7 @@ double	*xpos, *ypos;	/*o world coordinates (ra, dec) */
     int	ira, idec;
     double x, y, r, phi, theta, costhe, sinthe, dphi, cosphi, sinphi, dlng, z;
     double colatp, coslatp, sinlatp, longp;
-    double xs, ys, ra, dec;
+    double xs, ys, ra, dec, xp, yp;
     double wf_gseval();
 
     /* Convert from pixels to image coordinates */
@@ -221,9 +221,15 @@ double	*xpos, *ypos;	/*o world coordinates (ra, dec) */
     /*  Compute native spherical coordinates phi and theta in degrees from the
 	projected coordinates. this is the projection part of the computation */
     if (wcs->lngcor != NULL)
-	x = x + wf_gseval (wcs->lngcor, x, y);
+	xp = x + wf_gseval (wcs->lngcor, x, y);
+    else
+	xp = x;
     if (wcs->latcor != NULL)
-	y = y + wf_gseval (wcs->latcor, x, y);
+	yp = y + wf_gseval (wcs->latcor, x, y);
+    else
+	yp = y;
+    x = xp;
+    y = yp;
     r = sqrt (x * x + y * y);
 
     /* Compute phi */
@@ -504,7 +510,7 @@ struct WorldCoor *wcs;		/* pointer to the WCS descriptor */
 
 /* copyright(c) 1986 association of universities for research in astronomy inc.
  * wfgsurfit.x -- surface fitting package used by wcs function drivers.
- * Translated to C from SPP by Doug Mink, SAO, May 26, 1998
+ * Translated to C from SPP by Jessica Mink, SAO, May 26, 1998
  *
  * the following routines are used by the experimental function drivers tnx
  * and zpx to decode polynomial fits stored in the image header in the form
@@ -835,10 +841,10 @@ int	nxd, nyd;	/* order of the derivatives in x and y */
 	case TNX_XFULL:
 	    ptr2 = sf2->coeff + (sf2->yorder - 1) * sf2->xorder;
 	    ptr1 = coeff + (sf1->yorder - 1) * sf1->xorder;
-	    for (i = sf1->yorder - 1; i >= nyder+1; i--) {
+	    for (i = sf1->yorder - 1; i >= nyder; i--) {
 		for (j = i; j >= i-nyder+1; j--) {
 		    for (k = 0; k < sf2->xorder; k++)
-			ptr1[nxder+k] = ptr1[nxder+k] * (double)(j-1);
+			ptr1[nxder+k] = ptr1[nxder+k] * (double)(j);
 		    }
 		for (j = sf1->xorder; j >= nxder+1; j--) {
 		    for (k = j; k >= j-nxder+1; k--)
@@ -880,7 +886,7 @@ int	nxd, nyd;	/* order of the derivatives in x and y */
 
 	    else if (nxder > 0) { 
 		ptr1 = coeff;
-		ptr2 = sf2->coeff + sf2->ncoeff;
+		ptr2 = sf2->coeff + sf2->ncoeff - 1;
 		for (j = sf1->xorder; j >= nxder+1; j--) {
 		    for (k = j; k >= j - nxder + 1; k--)
 			ptr1[j-1] = ptr1[j-1] * (double)(k - 1);
@@ -890,10 +896,10 @@ int	nxd, nyd;	/* order of the derivatives in x and y */
 		}
 
 	    else if (nyder > 0) {
-		ptr1 = coeff + sf1->ncoeff;
+		ptr1 = coeff + sf1->ncoeff - 1;
 		ptr2 = sf2->coeff;
-		for (i = sf1->yorder; i > nyder + 1; i--) {
-		    for (j = i; j > i - nyder + 1; j--)
+		for (i = sf1->yorder; i >= nyder + 1; i--) {
+		    for (j = i; j >= i - nyder + 1; j--)
 			*ptr1 = *ptr1 * (double)(j - 1);
 		    ptr1 = ptr1 - 1;
 		    }
@@ -1218,4 +1224,11 @@ double	*coeff;
  * Jun 27 2005	Drop unused arguments k1 and k2 from wf_gsb1pol()
  *
  * Jan  8 2007	Drop unused variable ncoeff in wf_gsder()
+ * Jan  9 2007	Declare header const char in tnxinit()
+ * Apr  3 2007	Fix offsets to hit last cooefficient in wf_gsder()
+ *
+ * Sep  5 2008	Fix wf_gseval() call in tnxpos() so unmodified x and y are used
+ * Sep  9 2008	Fix loop in TNX_XFULL section of wf_gsder()
+ * 		(last two bugs found by Ed Los)
+ * Sep 17 2008	Fix tnxpos for null correction case (fix by Ed Los)
  */
